local AddonName, CAGI = ...
local ver -- assigned in addon loaded event

local on = true
local off = false

CAGI_Defaults = {
	Enable = on,
	Debug = off,
	Serious = off,
	Solo = on,
	Opp = on,
	RaidWarning = on,
	Mammoths = on,
	Links = on,
}
CAGI_Settings = {}

-- General settings
local LastGroupMembers = {}
local LastVersions = {}
local PChange = false
local ScanDelay = 3
local filter = {}
local filterTime = 4

CAGI.Frame = CreateFrame("Frame")
CAGI.Frame:RegisterEvent("ADDON_LOADED")
CAGI.Frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
CAGI.Frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
CAGI.Frame:RegisterEvent("CHAT_MSG_INSTANCE_CHAT")
CAGI.Frame:RegisterEvent("CHAT_MSG_INSTANCE_CHAT_LEADER")
CAGI.Frame:RegisterEvent("CHAT_MSG_PARTY")
CAGI.Frame:RegisterEvent("CHAT_MSG_PARTY_LEADER")
CAGI.Frame:RegisterEvent("CHAT_MSG_RAID")
CAGI.Frame:RegisterEvent("CHAT_MSG_RAID_LEADER")
CAGI.Frame:RegisterEvent("CHAT_MSG_RAID_WARNING")

-- Local text colors
local yel = "|cFFFFFF00"
local wht = "|cFFFFFFFF"
local blu = "|cFF00FFFF"
local red = "|cFFFF0000"
local grn = "|cFF00FF00"
local res = "|r"

-- Bill's Utils
local addOptionMt = BillsUtils.addOptionMt
local StatColor = BillsUtils.StatColor
local SJprint = BillsUtils.SJprint
local OptSaveTF = BillsUtils.OptSaveTF
local IsPvPZone = BillsUtils.IsPvPZone
local GenderWord = BillsUtils.GenderWord
local Wait = BillsUtils.Wait
local FLCap = BillsUtils.FLCap

BillsUtils.Locals[#BillsUtils.Locals +1] = function () --function is called if/when a newer version of Bill's Utils loads after the addon is loaded
	addOptionMt = BillsUtils.addOptionMt
	StatColor = BillsUtils.StatColor
	SJprint = BillsUtils.SJprint
	OptSaveTF = BillsUtils.OptSaveTF
	IsPvPZone = BillsUtils.IsPvPZone
	GenderWord = BillsUtils.GenderWord
	Wait = BillsUtils.Wait
	FLCap = BillsUtils.FLCap
end

-- values are as follows
--	1 = engineering bots and whatever
--	2 = rituals
--  3 = feasts
--  4 = cauldrons
--  5 = Battle Rezzes (rebirth/ Soulstone Ressurection / Raise Ally)
--  6 = reincarnation
--  7 = mammoths
--  8 = portals
--  9 = Soulstone predeath
-- 10 = Former Rituals
-- 11 = Lightwell
-- 12 = Demonic Gateway


local SpellList = {}
SpellList["COMBAT_LOG_EVENT_UNFILTERED"] = {}
SpellList["COMBAT_LOG_EVENT_UNFILTERED"]["SPELL_CAST_SUCCESS"] = {
	-- ENGINEERING BOTS GO HERE
	[54711] = 1,	--"Scrapbot"
	[54710] = 1,	--"MOLL-E"
	[67826] = 1,	--"Jeeves"
	[22700] = 1,	--"Field Repair Bot 74A"
	[44389] = 1,	--"Field Repair Bot 110G"
	
	-- Rituals
	[698] = 2,	--"Ritual of Summoning"
	
	-- Demonic Gateway
	[113902] = 12, -- "Demonic Gateway" 111771 ends as 113902
}

SpellList["COMBAT_LOG_EVENT_UNFILTERED"]["SPELL_CAST_START"] = {
	-- Conjures
	[29893] = 10,	--"Conjure Soulwell"
	[43987] = 10,	--"Conjure Refreshment"	
}

SpellList["COMBAT_LOG_EVENT_UNFILTERED"]["SPELL_CREATE"] = {
	-- FOOD ITEMS GO HERE
	[58465] = 3,	-- "Gigantic Feast"
	[58474] = 3,	-- "Small Feast"
	[87643] = 3,	-- "Broiled Dragon Feast"
	[87644] = 3,	-- "Seafood Magnifique Feast"
	[57426] = 3,	-- "Fish Feast"
	[57301] = 3,	-- "Great Feast"
	[66476] = 3,	-- "Bountiful Feast"
	[87915] = 3,	-- "Goblin Barbeque"
	-- New Panda Food follows
	[104958] = 3,	-- "Pandaren Banquet"
	[126503] = 3,	-- "Banquet of the Brew"
	[126492] = 3,	-- "Banquet of the Grill"
	[126501] = 3,	-- "Banquet of the Oven"
	[126497] = 3,	-- "Banquet of the Pot"
	[126499] = 3,	-- "Banquet of the Steamer"
	[126495] = 3,	-- "Banquet of the Wok"
	[105193] = 3,	-- "Great Pandaren Banquet"
	[126504] = 3,	-- "Great Banquet of the Brew"
	[126494] = 3,	-- "Great Banquet of the Grill"
	[126502] = 3,	-- "Great Banquet of the Oven"
	[126498] = 3,	-- "Great Banquet of the Pot"
	[126500] = 3,	-- "Great Banquet of the Steamer"
	[126496] = 3,	-- "Great Banquet of the Wok"
	
	-- CAULDRONS GO HERE
	[92654] = 4,	-- "Big Cauldron of Battle"
	[92692] = 4,	-- "Big Cauldron of Battle"
	[92652] = 4,	-- "Big Cauldron of Battle"
	[92693] = 4,	-- "Big Cauldron of Battle"
	[92635] = 4,	-- "Cauldron of Battle"
	[92689] = 4,	-- "Cauldron of Battle"
	[92645] = 4,	-- "Cauldron of Battle"
	[92691] = 4,	-- "Cauldron of Battle"
	[41443] = 4,	-- "Cauldron of Major Arcane Protection"
	[41494] = 4,	-- "Cauldron of Major Fire Protection"
	[41495] = 4,	-- "Cauldron of Major Frost Protection"
	[41497] = 4,	-- "Cauldron of Major Nature Protection"
	[41498] = 4,	-- "Cauldron of Major Shadow Protection"
}

SpellList["COMBAT_LOG_EVENT_UNFILTERED"]["SPELL_RESURRECT"] = {
	[20484] = 5,	-- REBIRTH
	[95750] = 5,	-- Soulstone Resurrection (battle rez)
	[61999] = 5,	-- Raise Ally ... I hope it is the right event type
	[20608] = 6,	-- REINCARNATION
}

SpellList["COMBAT_LOG_EVENT_UNFILTERED"]["SPELL_AURA_APPLIED"] = {
	[20707] = 9,	--"Soulstone Resurrection" pre death
}

SpellList["UNIT_SPELLCAST_SUCCEEDED"] = {
	-- Traveler's Tundra Mammoth
	[61447] = 7,	--"Traveler's Tundra Mammoth-Horde"
	[61425] = 7,	--"Traveler's Tundra Mammoth-Alliance"
	
	--Portals go here
	[11419] = 8,	--"Portal: Darnassus"
	[32266] = 8,	--"Portal: Exodar"
	[11416] = 8,	--"Portal: Ironforge"
	[10059] = 8,	--"Portal: Stormwind"
	[49360] = 8,	--"Portal: Theramore"
	[11417] = 8,	--"Portal: Orgrimmar"
	[32267] = 8,	--"Portal: Silvermoon"
	[11420] = 8,	--"Portal: Thunder Bluff"
	[11418] = 8,	--"Portal: Undercity"
	[49361] = 8,	--"Portal: Stonard"
	[88345] = 8,	--"Portal: Tol Barad - Alliance"
	[99346] = 8,	--"Portal: Tol Barad - Horde"
	
	--Other Stuff
	[724]	= 11,	--"Lightwell"

}

local e = 0
CAGI.Frame:SetScript("OnUpdate", function(self, elapsed)
	e = e + elapsed
	if e < 1/2 then return end
	
	if #filter > 0 then
		for x = #filter, 1, -1 do
			filter[x].expires = filter[x].expires - e
			if filter[x].expires <= 0 then
				table.remove(filter, x)
			end
		end
	end
	e = 0
end)
	
CAGI.Frame:SetScript("OnEvent",function(self, event, ...)
	if event == "ADDON_LOADED" then
		local arg = ...
		if arg == AddonName then
			addOptionMt( CAGI_Settings , CAGI_Defaults )
			ver = GetAddOnMetadata( AddonName, "Version")
			SJprint("\n", grn, "Come And Get It v.", ver, "loaded\n", res)
		end
		return self, event, ...
	end
	
	if CAGI_Settings.Debug then
		if type(...) == "table" then
			print("CAGI Debug: ", event,  unpack(...))
		else
			print("CAGI Debug: ", event, ... )
		end
	end
	
	if string.find(event, "^CHAT_MSG_") then
		local msg = ...
		if string.find(msg, "^CAGI:") then
			local _, caster, rest = string.split(" ", msg, 3)
			local found, _, spellId = string.find( rest, "spell:(%d+)" )
			if found then 
				filter[#filter+1] = {expires = filterTime, id = tonumber(spellId), by = caster}
			end
		end
		return
	end
	
	local etype, sourceName, destName, spellId, spellName, unit, spellType
	if event == "UNIT_SPELLCAST_SUCCEEDED" then
		unit, spellName, _, _, spellId = ...
		if not( SpellList[event][spellId] ) then return end
		spellType = SpellList[event][spellId]
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
		_, etype, _, _, sourceName, _, _, _, destName, _, _, spellId, spellName = ...
		if not( SpellList[event][etype] ) or not( SpellList[event][etype][spellId] ) then return end
		spellType = SpellList[event][etype][spellId]
	else
		return
	end
	if not(sourceName) then 
		sourceName = UnitName(unit)
	end
	Wait( (math.random( 250,1000 ) / 1000), CAGI.MsgOut, sourceName, spellName, spellId, spellType, destName)
end)

--output the msg to the group here
CAGI.MsgOut = function(sourceName, spellName, spellId, msgType, destName)
	if #filter > 0 then
		for x = 1, #filter do
			if spellId == filter[x].id then
				local by = string.split("-", sourceName)
				if filter[x].by == by then
					return
				end
			end
		end
	end
	
	filter[#filter+1] = {expires = filterTime, id = tonumber(spellId), by = (string.split("-", sourceName))}
	
	local spellLink = false
	local name = sourceName
	if string.find(name," ") ~= nil then
		name = string.split(" ", name)
	end
	if string.find(name,"-") ~= nil then
		name = string.split("-", name)
	end
	
	if CAGI_Settings.Links then
		spellLink = "|cff71d5ff|Hspell:"..spellId.."|h["..spellName.."]|h|r"
	end

	local InGroup = UnitInParty(sourceName) or UnitInRaid(sourceName) or UnitInBattleground(sourceName) or UnitIsPlayer(sourceName)
	InGroup = InGroup or UnitInParty(name) or UnitInRaid(name) or UnitInBattleground(name) or UnitIsPlayer( name )
	
	if msgType == 7 and not( CAGI_Settings.Mammoths ) then return end
	if msgType == 7 and not(InGroup) then return end -- opp side mammoths... cant use em so who cares
	if not( InGroup ) and not( CAGI_Settings.Opp ) then return end

	local OutputMsg
	if InGroup and CAGI_Settings.Serious then
		OutputMsg = name.." casts/made/creates a "..(spellLink and spellLink or spellName)
					
	elseif InGroup then
		if msgType == 1 then -- engineering bots
			local PrefixTable = {"built a", "assembled together a", "hammered together a", "assembled "..GenderWord(sourceName, "himself").." a" }
			local SuffixTable = {"Beware of sharp edges.", "A hammer and duct tape can be amazing.", "I think there is a screw loose.", "I got hit by a cog!", "Why are there so many extra parts?"}
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
			
		elseif msgType == 2 then -- rituals
			local PrefixTable = {"is making a portal for a", "is trying to tear a hole in space for a", "is casting the fabled" }
			local SuffixTable = {"Clickey Clickey!", "Lend a hand or paw.", "Just Click it.", "Help "..GenderWord(sourceName, "him" ).." summon it up."}
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
			
		elseif msgType == 3 then -- feasts
			local PrefixTable = {"cooked up a nice" , "made a nice steaming", "made a big spread of", "made a tastey", "may have burned the", "made "..GenderWord(sourceName, "his").." special"}
			local SuffixTable = {"Dig in!", "Chow down!", "Thanks for the grub.", "Free Food!", "It tastes like chicken!", "Thank "..GenderWord(sourceName, "him").." for the food!"}
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
			
		elseif msgType == 4 then -- cauldrons
			local PrefixTable = {"brewed up a", "laid out a bubbling", "mixed up a", "brewed a bubbling"}
			local SuffixTable = {"Better living through chemistry.", "The bubbles tickle my nose.", "It may taste bad, but it's good for you.", "This is not your mother's medicine"}
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
			
		elseif msgType == 5 then -- Battle rezzes  (except reincarnation)
			OutputMsg = name.." casts "..(spellLink and spellLink or spellName).." on "..destName
			
		elseif msgType == 6 then -- reincarnation
			OutputMsg = name.." casts "..(spellLink and spellLink or spellName).." on "..GenderWord(sourceName, "himself" )
			
		elseif msgType == 7 then -- mammoths
			local PrefixTable = {"is riding on", "is mounted on", "climbed up on", "mounted", "summoned" }
			local SuffixTable = {"Hi ho Silver, Away!", "For your safety, stand to the front of the mount.", "Toxic exhaust!", "Vendors now open for business.", "Bet "..GenderWord( sourceName, "he").." wishes "..GenderWord(sourceName, "he").." got a cut from the vendors."}
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..GenderWord( sourceName, "his").." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
			
		elseif msgType == 8 then -- portals
			local PrefixTable = {"is casting", "is ripping a hole in space for a" }
			local SuffixTable = {"Make your escape!", "See new and exciting places.", "Have mage, Will Travel.", "Let's thank "..GenderWord( sourceName, "him") }
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
		
		elseif msgType == 9 then -- Soulstone Resurrection applied pre death
			if sourceName == destName then
				OutputMsg = name.." casts "..(spellLink and spellLink or spellName).." on "..GenderWord( sourceName, "himself" ).." "..FLCap(GenderWord( sourceName, "he" )).." is now protected for the next 15 minutes."
			else
				OutputMsg = name.." casts "..(spellLink and spellLink or spellName).." on "..destName..". "..FLCap(GenderWord(destName, "he")).." is now protected for the next 15 minutes"
			end
			
		elseif msgType == 10 then -- Conjures
			local PrefixTable = {"is casting", "is ripping a hole in space for", "is casting the fabled"  }
			local SuffixTable = {"No help needed.", "Free conjured stuff!", "Come and get it.", "Let's thank "..GenderWord( sourceName, "him"), "Remember when it took clicks to help?" }
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
			
		elseif msgType == 11 then -- Lightwell
			local PrefixTable = {"is making a", "summoned up a", "is casting the fabled"  }
			local SuffixTable = {"Injured? Then click it.", "Now you can help heal yourself", "Don't forget to click it", "Let's thank "..GenderWord( sourceName, "him"), "Take the gift of life." }
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
			
		elseif msgType == 12 then -- Demonic Gateway
			local PrefixTable = {"summoned up a", "casted "..GenderWord( sourceName, "his" ).." fabled"  }
			local SuffixTable = {"A shortcut through space and time.", "Don't get stuck between gateways.", "Sure beats walking.", (CAGI_Settings.Links and "If you have a |cffffffff|Hitem:31167|h[Gateway Control Stone]|h|r you won't even have to click." or "If you have a Gateway Control Stone you won't even have to click."), }
			OutputMsg = name.." "..PrefixTable[math.random(1, #PrefixTable)].." "..(spellLink and spellLink or spellName)..". "..SuffixTable[math.random(1, #SuffixTable)]
		else -- catch all
			return
		end
	else
		if destName and destName ~= name and destName ~= spellName then
			OutputMsg = "Opposing Side Alert: "..name.." casts "..(spellLink and spellLink or spellName).." on "..destName
		else
			OutputMsg = "Opposing Side Alert: "..name.." casts "..(spellLink and spellLink or spellName)
		end
	end
		
	local mode = false
	local _, IType = IsInInstance()
	
	if HasLFGRestrictions() then
		mode = "INSTANCE_CHAT"
	elseif IType == "pvp" then
		mode = "INSTANCE_CHAT" -- "BATTLEGROUND"
	elseif IType == "arena" then
		mode = "PARTY"
	elseif IsInRaid() then
		if CAGI_Settings.RaidWarning and (UnitIsGroupLeader("player") or UnitIsGroupAssistant("player")) then
			mode = "RAID_WARNING"
		else
			mode = "RAID"
		end
	elseif GetNumSubgroupMembers() > 0 then
		mode = "PARTY"
	end
	if CAGI_Settings.Debug or CAGI_Settings.Solo then
		print("CAGI: "..OutputMsg )
	end
	
	if mode then
		SendChatMessage( "CAGI: "..OutputMsg, mode )
	end
	
	
end

SLASH_CAGI1 = "/cagi"
SlashCmdList["CAGI"] = function(msg)
	local cmd,arg = string.split(" ", msg)
	cmd = cmd:lower()
	
	if cmd =="config" then
		InterfaceOptionsFrame_OpenToCategory( CAGI.panel )
		return
	end
	
	if not(CAGI_Settings.Enable) and cmd ~= "enable" then
		print(" ")
		SJprint( yel, "Come And Get It", res)
		SJprint( yel, "  This addon is ", red, "DISABLED", res)
		SJprint( yel, "  Type: ", blu, "/cagi enable on", res)
		SJprint( yel, "  to enable.", res)
		return
	end

	local Command = {"Enable", "Debug", "Serious", "Solo", "Opp", "RaidWarning", "Mammoths", "Links" }
	
	for x = 1, #Command do
		if cmd == string.lower(Command[x]) then
			OptSaveTF( CAGI_Settings, Command[x], arg)
			SJprint( yel, "Come And Get It:", Command[x],"is", StatColor( CAGI_Settings[Command[x]] ), res)
			return
		end
	end

	if cmd == "" then
		print(" ")
		SJprint( yel, "________________________________________", res)
		SJprint( yel, "Come And Get It", res)
		SJprint( yel, "  Addon State:", res)
		for x = 1, #Command do
			SJprint( blu, Command[x], wht, "is", StatColor( CAGI_Settings[Command[x]] ), res)
		end
		SJprint( yel, "Available commands are listed above in", blu, "BLUE.", res)
		SJprint( yel, "Type", blu, "/cagi config", yel, "to open the GUI config screen.", res)
		SJprint( yel, "Type", blu, "/cagi cmdlist", yel, "for a description of commands.", res)
		SJprint( yel, "________________________________________", res)
		return
	end

	if cmd == "cmdlist" or cmd == "help" then
		print(" ")
		SJprint( yel, "________________________________________", res)
		SJprint( yel, "Come And Get It", res)
		SJprint( yel, "  Addon State:", res)
		SJprint( blu, "     Enable", wht, "Turns the Add On on and off", res)
		SJprint( blu, "     Debug", wht, "Turns debug messages on and off", res)
		SJprint( blu, "     Serious", wht, "Turns Serious/Strict messages on and off", res)
		SJprint( blu, "     Solo", wht, "Enable announcements to yourself on and off", res)
		SJprint( blu, "     Opp", wht, "Enable opposing side announcements on and off", res)
		SJprint( blu, "     RaidWarning", wht, "Enable announcements in RaidWarnings if you can use them", res)
		SJprint( blu, "     Mammoths", wht, "Enable announcements when someone mounts a Traveler's Tundra Mammoth", res)
		SJprint( blu, "     Links", wht, "Output detected items as a link instead of just text", res)
		SJprint( yel, "Available commands are listed above in ", blu, "BLUE.", res)
		SJprint( yel, "Type ", blu, "/cagi config",  yel, " to open the GUI config screen.", res)
		SJprint( yel, "________________________________________", res)
		return
	end

	SJprint( yel, "  Come And Get It", res)
	SJprint( yel, "  Error: ", red, cmd, yel, " is not a valid command", res)
	SJprint( yel, "  Type: ", blu, "/cagi", yel, "for the addon status and command list", res)
	SJprint( yel, "  Type: ", blu, "/cagi cmdlist", yel, "for command list help", res)
	return
end
